"use strict";

/* exported logger */

var GuiConnector = require('../../GuiConnector');
var AbstractGuiElement = require('../AbstractGuiElement');
var Functions = require('../../Functions');

var logger = require('../../logger');

function GuiUtils(configuration) {
  var self = this;
  self.setConfiguration(configuration);
}

GuiUtils.prototype = Object.create(AbstractGuiElement.prototype);
GuiUtils.prototype.constructor = GuiUtils;

GuiUtils.prototype.setConfiguration = function(configuration) {
  this._configuration = configuration;
};

GuiUtils.prototype.getConfiguration = function() {
  return this._configuration;
};

GuiUtils.prototype.createLabel = function(value) {
  var result = document.createElement("span");
  result.innerHTML = value;
  result.className = "minerva-label";
  return result;
};

GuiUtils.prototype.createPostTranslationalModifications = function(label, value) {
  var result = document.createElement("div");
  if (value !== undefined && value.length > 0) {
    var self = this;
    result.appendChild(self.createLabel(label));
    result.appendChild(self.createNewLine());
    var list = Functions.createElement({
      type : "ul"
    });
    for (var i = 0; i < value.length; i++) {
      var name = value[i].name;
      var modificationStateType = self.getConfiguration().getModificationStateTypeByName(value[i].state);
      var state = modificationStateType.getCommonName();
      var desc;
      if (name !== null && name !== undefined && name !== "") {
        desc = state + " at position " + name + ", ";
      } else {
        desc = state + ",";
      }
      var row = Functions.createElement({
        type : "li"
      });
      row.appendChild(self.createLabelText(desc));
      list.appendChild(row);
    }
    result.appendChild(list);
  }
  return result;
};

GuiUtils.prototype.createCandidates = function(label, value) {
  var result = document.createElement("div");
  if (value !== undefined) {
    throw new Error("Not implemented");
  }
  return result;
};

GuiUtils.prototype.createChebiTree = function(label, value) {
  var result = document.createElement("div");
  if (value !== undefined) {
    throw new Error("Not implemented");
  }
  return result;
};
GuiUtils.prototype.createSeparator = function() {
  var result = document.createElement("hr");
  return result;
};

GuiUtils.prototype.createNewLine = function(count) {
  var result = document.createElement("p");
  if (count > 0) {
    result.style.height = ((count - 1) * 10) + "px";
  }
  return result;
};

GuiUtils.prototype.createLink = function(url, name) {
  var link = document.createElement("a");
  link.href = url;
  link.innerHTML = name;
  link.target = "_blank";
  link.style.textDecoration = "underline";
  return link;
};

GuiUtils.prototype.createAnnotationLink = function(annotation, showType) {
  var self = this;
  var name, type, hint;
  var article = annotation.getArticle();
  if (article !== undefined) {

    hint = article.getTitle() + " " + article.getAuthors().join(", ") + ", " + article.getYear() + ", "
        + article.getJournal();
    type = "PUBMED";
    name = article.getId();
  } else {
    name = annotation.getResource();
    var miriamType = self.getConfiguration().getMiriamTypeByName(annotation.getType());
    if (miriamType === null) {
      logger.warn("Unknown miriam type: " + annotation.getType());
      type = annotation.getType();
    } else {
      type = miriamType.getCommonName();
    }
  }
  var link;
  if (showType) {
    link = self.createLink(annotation.getLink(), type + " (" + name + ")");
  } else {
    link = self.createLink(annotation.getLink(), name);
  }
  if (hint !== undefined) {
    var div = document.createElement("div");
    div.title = hint;
    div.appendChild(link);
    return div;
  } else {
    return link;
  }
};

GuiUtils.prototype.createAnnotations = function(label, value, options) {
  var self = this;

  var inline = false;
  if (options !== undefined) {
    if (options.inline !== undefined) {
      inline = options.inline;
    }
  }
  var result = document.createElement("div");
  result.appendChild(self.createLabel(label));
  if (value !== undefined && value.length > 0) {
    if (!inline) {
      result.appendChild(self.createNewLine());
    }
    result.appendChild(self.createAnnotationList(value, options));
  } else {
    result.appendChild(self.createLabelText("No annotations"));
  }
  return result;
};

GuiUtils.prototype.createAnnotationList = function(annotations, options) {
  var showType = true;
  var inline = false;
  if (options !== undefined) {
    if (options.showType !== undefined) {
      showType = options.showType;
    }
    if (options.inline !== undefined) {
      inline = options.inline;
    }
  }

  var self = this;
  var result = document.createElement("div");
  for (var i = 0; i < annotations.length; i++) {
    var element = annotations[i];
    var link = self.createAnnotationLink(element, showType);
    if (inline) {
      if (i > 0) {
        var coma = document.createElement("span");
        coma.innerHTML = ", ";
        result.appendChild(coma);
      }
      result.appendChild(link);
    } else {

      var row = document.createElement("div");
      row.style.height = "26px";
      if (i % 2 === 0) {
        row.className = "minerva-annotation-row-odd";
      } else {
        row.className = "minerva-annotation-row-even";
      }
      var header = document.createElement("div");
      header.style.width = "24px";
      header.style.float = "left";
      header.innerHTML = "[" + (i + 1) + "]";
      row.appendChild(header);

      var body = document.createElement("div");
      body.style.float = "left";
      body.appendChild(link);
      row.appendChild(body);
      result.appendChild(row);
    }
  }
  return result;
};

GuiUtils.prototype.createLabelText = function(value) {
  var result = document.createElement("span");
  if (value !== undefined) {
    result.innerHTML = value;
  }
  return result;
};

GuiUtils.prototype.createInputText = function(value) {
  var result = document.createElement("input");
  result.setAttribute('type', 'text');

  if (value !== undefined) {
    result.setAttribute('value', value);
  }
  return result;
};

GuiUtils.prototype.createTextArea = function(value) {
  var result = document.createElement("textarea");

  if (value !== undefined) {
    result.setAttribute('value', value);
    result.innerHTML = value;
  }
  return result;
};

GuiUtils.prototype.createParamLine = function(label, value) {
  var result = document.createElement("div");
  if (value !== undefined && value !== null && value !== "") {
    var self = this;
    result.appendChild(self.createLabel(label));
    result.appendChild(self.createLabelText(value));
  }
  return result;
};

GuiUtils.prototype.createIcon = function(icon) {
  var result = document.createElement("div");
  if (icon !== undefined && icon !== null) {
    var img = document.createElement("img");
    img.src = GuiConnector.getImgPrefix() + icon;
    img.style.float = "left";
    img.hspace = "5";
    result.appendChild(img);
  }
  return result;
};

GuiUtils.prototype.createArrayParamLine = function(label, value) {
  var result = document.createElement("div");
  if (value !== undefined && value.length > 0) {
    var self = this;
    result.appendChild(self.createLabel(label));
    result.appendChild(self.createLabelText(value.join(", ")));
  }
  return result;
};

GuiUtils.prototype.createSubMapLink = function(label, modelId) {
  var self = this;
  var result = document.createElement("div");
  if (modelId !== undefined) {
    var button = document.createElement("button");
    var model = self.getMap().getSubmapById(modelId).getModel();

    button.innerHTML = model.getName();
    button.onclick = function() {
      return self.getMap().openSubmap(modelId);
    };
    result.appendChild(this.createLabel(label));
    result.appendChild(button);
  }
  return result;
};

GuiUtils.prototype.createTableRow = function(elements) {
  var row = Functions.createElement({
    type : "div",
    style : "display: table-row;"
  });

  for (var i = 0; i < elements.length; i++) {
    var cell = Functions.createElement({
      type : "div",
      style : "display: table-cell;"
    });
    cell.appendChild(elements[i]);
    row.appendChild(cell);
  }
  return row;
};

GuiUtils.prototype.createReactionElement = function(params) {
  var reaction = params.reaction;
  var showTitle = ((params.showTitle === undefined) || params.showTitle);
  var self = this;
  var div = document.createElement("div");

  if (showTitle) {
    div.appendChild(self.createLabel("Reaction: " + reaction.getReactionId()));
    if (reaction.getModelId() !== self.getMap().getId()) {
      div.appendChild(self.createSubMapLink("In submodel: ", reaction.getModelId()));
    }
  }

  if (reaction.getLinkedSubmodelId() !== null && reaction.getLinkedSubmodelId() !== undefined) {
    div.appendChild(self.createSubMapLink("Associated submodel: ", reaction.getLinkedSubmodelId()));
  }
  if (showTitle) {
    div.appendChild(self.createNewLine());
  }

  div.appendChild(self.createParamLine("Type: ", reaction.getType()));
  div.appendChild(self.createParamLine("Symbol: ", reaction.getSymbol()));
  div.appendChild(self.createParamLine("Abbreviation: ", reaction.getAbbreviation()));
  div.appendChild(self.createParamLine("Formula: ", reaction.getFormula()));
  div.appendChild(self.createParamLine("Mechanical Confidence Score: ", reaction.getMechanicalConfidenceScore()));
  div.appendChild(self.createParamLine("Lower Bound: ", reaction.getLowerBound()));
  div.appendChild(self.createParamLine("Upper Bound: ", reaction.getUpperBound()));
  div.appendChild(self.createParamLine("Gene Protein Reaction: ", reaction.getGeneProteinReaction()));
  div.appendChild(self.createParamLine("Subsystem: ", reaction.getSubsystem()));
  div.appendChild(self.createArrayParamLine("Synonyms: ", reaction.getSynonyms()));
  div.appendChild(self.createParamLine("Description: ", reaction.getDescription()));
  div.appendChild(self.createReactantsLine(reaction.getReactants()));
  div.appendChild(self.createProductsLine(reaction.getProducts()));
  div.appendChild(self.createModifiersLine(reaction.getModifiers()));
  div.appendChild(self.createCandidates("Candidates: ", reaction.getOther('dataMining')));
  div.appendChild(self.createAnnotations("Annotations: ", reaction.getReferences()));
  return div;
};

GuiUtils.prototype.createAliasElement = function(params) {
  var alias = params.alias;
  var icon = params.icon;
  var showTitle = ((params.showTitle === undefined) || params.showTitle);
  var self = this;
  var div = document.createElement("div");

  if (showTitle) {
    if (icon !== undefined) {
      div.appendChild(this.createIcon(icon));
    }

    div.appendChild(this.createParamLine(alias.getType() + ": ", alias.getName()));
    if (alias.getModelId() !== self.getMap().getId()) {
      div.appendChild(self.createSubMapLink("In submodel: ", alias.getModelId()));
    }
  }
  if (alias.getLinkedSubmodelId() !== null && alias.getLinkedSubmodelId() !== undefined) {
    div.appendChild(self.createSubMapLink("Associated submodel: ", alias.getLinkedSubmodelId()));
  }
  if (showTitle) {
    div.appendChild(self.createNewLine(3));
  }

  div.appendChild(self.createParamLine("Full name: ", alias.getFullName()));
  div.appendChild(self.createParamLine("Symbol: ", alias.getSymbol()));
  div.appendChild(self.createParamLine("Abbreviation: ", alias.getAbbreviation()));
  div.appendChild(self.createParamLine("Formula: ", alias.getFormula()));
  div.appendChild(self.createArrayParamLine("Former symbols: ", alias.getFormerSymbols()));
  div.appendChild(self.createPostTranslationalModifications("PostTranslational modifications: ", alias
      .getOther('modifications')));
  div.appendChild(self.createParamLine("Charge: ", alias.getCharge()));
  div.appendChild(self.createArrayParamLine("Synonyms: ", alias.getSynonyms()));
  div.appendChild(self.createLabelText(alias.getDescription()));
  div.appendChild(self.createCandidates("Candidates: ", alias.getOther('dataMining')));
  div.appendChild(self.createChebiTree("Chebi ontology: ", alias.getOther('chebiTree')));
  div.appendChild(self.createAnnotations("Annotations: ", alias.getReferences()));
  return div;
};

GuiUtils.prototype.createReactantsLine = function(label, value) {
  var result = document.createElement("div");
  if (value !== undefined && value.length > 0) {
    for (var i = 0; i < value.length; i++) {
      result.appendChild(this.createParamLine("Reactant: ", value[i]));
    }
  }
  return result;
};

GuiUtils.prototype.createProductsLine = function(label, value) {
  var result = document.createElement("div");
  if (value !== undefined && value.length > 0) {
    for (var i = 0; i < value.length; i++) {
      result.appendChild(this.createParamLine("Product: ", value[i]));
    }
  }
  return result;
};

GuiUtils.prototype.createModifiersLine = function(label, value) {
  var result = document.createElement("div");
  if (value !== undefined && value.length > 0) {
    for (var i = 0; i < value.length; i++) {
      result.appendChild(this.createParamLine("Modifier: ", value[i]));
    }
  }
  return result;
};

GuiUtils.prototype.createTabMenuObject = function(params) {
  var name = params.name;
  var id = params.id;
  var navigationBar = params.navigationBar;

  var navClass = '';
  if (navigationBar.children.length === 0) {
    navClass = "active";
  }

  var navLi = document.createElement("li");
  navLi.className = navClass;

  var navLink = document.createElement("a");
  navLink.href = "#" + id;
  if (name !== undefined) {
    navLink.innerHTML = name;
  }
  navLink.onclick = function() {
    $(this).tab('show');
  };
  navLi.appendChild(navLink);
  if (name !== undefined) {
    navLink.innerHTML = name;
  }
  return navLi;
};

GuiUtils.prototype.createTabContentObject = function(params) {
  var navigationObject = params.navigationObject;
  var tabId = params.id;
  var result = document.createElement("div");
  result.style.height = "100%";

  var contentClass = 'tab-pane';
  if (navigationObject.className === "active") {
    contentClass = "tab-pane active";
  }

  result.className = contentClass;
  result.id = tabId;
  return result;
};

module.exports = GuiUtils;
